@@ -61,59 +61,34 @@ void
 free_tlvs (struct tlvs *tlvs)
 {
   if (tlvs->area_addrs)
-    {
-      list_delete (tlvs->area_addrs);
-    }
+    list_delete (tlvs->area_addrs);
   if (tlvs->is_neighs)
-    {
-      list_delete (tlvs->is_neighs);
-    }
+    list_delete (tlvs->is_neighs);
   if (tlvs->te_is_neighs)
-    {
-      list_delete (tlvs->te_is_neighs);
-    }
+    list_delete (tlvs->te_is_neighs);
   if (tlvs->es_neighs)
-    {
-      list_delete (tlvs->es_neighs);
-    }
+    list_delete (tlvs->es_neighs);
   if (tlvs->lsp_entries)
-    {
-      list_delete (tlvs->lsp_entries);
-    }
+    list_delete (tlvs->lsp_entries);
   if (tlvs->lan_neighs)
-    {
-      list_delete (tlvs->lan_neighs);
-    }
+    list_delete (tlvs->lan_neighs);
   if (tlvs->prefix_neighs)
-    {
-      list_delete (tlvs->prefix_neighs);
-    }
+    list_delete (tlvs->prefix_neighs);
   if (tlvs->ipv4_addrs)
-    {
-      list_delete (tlvs->ipv4_addrs);
-    }
+    list_delete (tlvs->ipv4_addrs);
   if (tlvs->ipv4_int_reachs)
-    {
-      list_delete (tlvs->ipv4_int_reachs);
-    }
+    list_delete (tlvs->ipv4_int_reachs);
   if (tlvs->ipv4_ext_reachs)
-    {
-      list_delete (tlvs->ipv4_ext_reachs);
-    }
+    list_delete (tlvs->ipv4_ext_reachs);
   if (tlvs->te_ipv4_reachs)
-    {
-      list_delete (tlvs->te_ipv4_reachs);
-    }
+    list_delete (tlvs->te_ipv4_reachs);
 #ifdef HAVE_IPV6
   if (tlvs->ipv6_addrs)
-    {
-      list_delete (tlvs->ipv6_addrs);
-    }
+    list_delete (tlvs->ipv6_addrs);
   if (tlvs->ipv6_reachs)
-    {
-      list_delete (tlvs->ipv6_reachs);
-    }
+    list_delete (tlvs->ipv6_reachs);
 #endif /* HAVE_IPV6 */
+  
   return;
 }
 
@@ -796,7 +771,7 @@ err:
 int
 tlv_add_is_neighs (struct list *is_neighs, struct stream *stream)
 {
-  struct listnode *node, *nnode;
+  struct listnode *node;
   struct is_neigh *is_neigh;
   u_char value[255];
   u_char *pos = value;
@@ -805,7 +780,7 @@ tlv_add_is_neighs (struct list *is_neighs, struct stream *stream)
   *pos = 0;			/*is_neigh->virtual; */
   pos++;
 
-  for (ALL_LIST_ELEMENTS (is_neighs, node, nnode, is_neigh))
+  for (ALL_LIST_ELEMENTS_RO (is_neighs, node, is_neigh))
     {
       if (pos - value + IS_NEIGHBOURS_LEN > 255)
 	{
@@ -829,16 +804,48 @@ tlv_add_is_neighs (struct list *is_neighs, struct stream *stream)
   return add_tlv (IS_NEIGHBOURS, pos - value, value, stream);
 }
 
+int
+tlv_add_te_is_neighs (struct list *te_is_neighs, struct stream *stream)
+{
+  struct listnode *node;
+  struct te_is_neigh *te_is_neigh;
+  u_char value[255];
+  u_char *pos = value;
+  int retval;
+
+  for (ALL_LIST_ELEMENTS_RO (te_is_neighs, node, te_is_neigh))
+    {
+      /* FIXME: This will be wrong if we are going to add TE sub TLVs. */
+      if (pos - value + IS_NEIGHBOURS_LEN > 255)
+        {
+          retval = add_tlv (TE_IS_NEIGHBOURS, pos - value, value, stream);
+          if (retval != ISIS_OK)
+            return retval;
+          pos = value;
+        }
+      
+      memcpy (pos, te_is_neigh->neigh_id, ISIS_SYS_ID_LEN + 1);
+      pos += ISIS_SYS_ID_LEN + 1;
+      memcpy (pos, te_is_neigh->te_metric, 3);
+      pos += 3;
+      /* Sub TLVs length. */
+      *pos = 0;
+      pos++;
+    }
+
+  return add_tlv (TE_IS_NEIGHBOURS, pos - value, value, stream);
+}
+
 int
 tlv_add_lan_neighs (struct list *lan_neighs, struct stream *stream)
 {
-  struct listnode *node, *nnode;
+  struct listnode *node;
   u_char *snpa;
   u_char value[255];
   u_char *pos = value;
   int retval;
 
-  for (ALL_LIST_ELEMENTS (lan_neighs, node, nnode, snpa))
+  for (ALL_LIST_ELEMENTS_RO (lan_neighs, node, snpa))
     {
       if (pos - value + ETH_ALEN > 255)
 	{
@@ -854,24 +861,9 @@ tlv_add_lan_neighs (struct list *lan_neighs, struct stream *stream)
   return add_tlv (LAN_NEIGHBOURS, pos - value, value, stream);
 }
 
-/*
-  u_char value[255];
-  u_char *pos = value;
-
-  if (circuit->ip_router) {                             
-    *pos =  (u_char)NLPID_IP;
-    pos ++;
-  }
-  if (circuit->ipv6_router) {                         
-    *pos = (u_char)NLPID_IPV6;
-    pos ++;
-  }
-*/
-
 int
 tlv_add_nlpid (struct nlpids *nlpids, struct stream *stream)
 {
-
   return add_tlv (PROTOCOLS_SUPPORTED, nlpids->count, nlpids->nlpids, stream);
 }
 
@@ -898,13 +890,13 @@ tlv_add_checksum (struct checksum *checksum, struct stream *stream)
 int
 tlv_add_ip_addrs (struct list *ip_addrs, struct stream *stream)
 {
-  struct listnode *node, *nnode;
+  struct listnode *node;
   struct prefix_ipv4 *ipv4;
   u_char value[255];
   u_char *pos = value;
   int retval;
 
-  for (ALL_LIST_ELEMENTS (ip_addrs, node, nnode, ipv4))
+  for (ALL_LIST_ELEMENTS_RO (ip_addrs, node, ipv4))
     {
       if (pos - value + IPV4_MAX_BYTELEN > 255)
 	{
@@ -920,6 +912,20 @@ tlv_add_ip_addrs (struct list *ip_addrs, struct stream *stream)
   return add_tlv (IPV4_ADDR, pos - value, value, stream);
 }
 
+/* Used to add TLV containing just one IPv4 address - either IPv4 address TLV
+ * (in case of LSP) or TE router ID TLV. */
+int
+tlv_add_in_addr (struct in_addr *addr, struct stream *stream, u_char tag)
+{
+  u_char value[255];
+  u_char *pos = value;
+  
+  memcpy (pos, addr, IPV4_MAX_BYTELEN);
+  pos += IPV4_MAX_BYTELEN;
+
+  return add_tlv (tag, pos - value, value, stream);
+}
+
 int
 tlv_add_dynamic_hostname (struct hostname *hostname, struct stream *stream)
 {
@@ -930,13 +936,13 @@ tlv_add_dynamic_hostname (struct hostname *hostname, struct stream *stream)
 int
 tlv_add_lsp_entries (struct list *lsps, struct stream *stream)
 {
-  struct listnode *node, *nnode;
+  struct listnode *node;
   struct isis_lsp *lsp;
   u_char value[255];
   u_char *pos = value;
   int retval;
 
-  for (ALL_LIST_ELEMENTS (lsps, node, nnode, lsp))
+  for (ALL_LIST_ELEMENTS_RO (lsps, node, lsp))
     {
       if (pos - value + LSP_ENTRIES_LEN > 255)
 	{
@@ -961,13 +967,13 @@ tlv_add_lsp_entries (struct list *lsps, struct stream *stream)
 int
 tlv_add_ipv4_reachs (struct list *ipv4_reachs, struct stream *stream)
 {
-  struct listnode *node, *nnode;
+  struct listnode *node;
   struct ipv4_reachability *reach;
   u_char value[255];
   u_char *pos = value;
   int retval;
 
-  for (ALL_LIST_ELEMENTS (ipv4_reachs, node, nnode, reach))
+  for (ALL_LIST_ELEMENTS_RO (ipv4_reachs, node, reach))
     {
       if (pos - value + IPV4_REACH_LEN > 255)
 	{
@@ -995,17 +1001,50 @@ tlv_add_ipv4_reachs (struct list *ipv4_reachs, struct stream *stream)
   return add_tlv (IPV4_INT_REACHABILITY, pos - value, value, stream);
 }
 
+int
+tlv_add_te_ipv4_reachs (struct list *te_ipv4_reachs, struct stream *stream)
+{
+  struct listnode *node;
+  struct te_ipv4_reachability *te_reach;
+  u_char value[255];
+  u_char *pos = value;
+  u_char prefix_size;
+  int retval;
+
+  for (ALL_LIST_ELEMENTS_RO (te_ipv4_reachs, node, te_reach))
+    {
+      prefix_size = ((((te_reach->control & 0x3F) - 1) >> 3) + 1);
+
+      if (pos - value + (5 + prefix_size) > 255)
+	{
+	  retval =
+	    add_tlv (IPV4_INT_REACHABILITY, pos - value, value, stream);
+	  if (retval != ISIS_OK)
+	    return retval;
+	  pos = value;
+	}
+      *(u_int32_t *) pos = te_reach->te_metric;
+      pos += 4;
+      *pos = te_reach->control;
+      pos++;
+      memcpy (pos, &te_reach->prefix_start, prefix_size);
+      pos += prefix_size;
+    }
+
+  return add_tlv (TE_IPV4_REACHABILITY, pos - value, value, stream);
+}
+
 #ifdef HAVE_IPV6
 int
 tlv_add_ipv6_addrs (struct list *ipv6_addrs, struct stream *stream)
 {
-  struct listnode *node, *nnode;
+  struct listnode *node;
   struct prefix_ipv6 *ipv6;
   u_char value[255];
   u_char *pos = value;
   int retval;
 
-  for (ALL_LIST_ELEMENTS (ipv6_addrs, node, nnode, ipv6))
+  for (ALL_LIST_ELEMENTS_RO (ipv6_addrs, node, ipv6))
     {
       if (pos - value + IPV6_MAX_BYTELEN > 255)
 	{
@@ -1024,13 +1063,13 @@ tlv_add_ipv6_addrs (struct list *ipv6_addrs, struct stream *stream)
 int
 tlv_add_ipv6_reachs (struct list *ipv6_reachs, struct stream *stream)
 {
-  struct listnode *node, *nnode;
+  struct listnode *node;
   struct ipv6_reachability *ip6reach;
   u_char value[255];
   u_char *pos = value;
   int retval, prefix_octets;
 
-  for (ALL_LIST_ELEMENTS (ipv6_reachs, node, nnode, ip6reach))
+  for (ALL_LIST_ELEMENTS_RO (ipv6_reachs, node, ip6reach))
     {
       if (pos - value + IPV6_MAX_BYTELEN + 6 > 255)
 	{
@@ -1069,7 +1108,7 @@ tlv_add_padding (struct stream *stream)
 	goto err;
       if (!stream_putc (stream, (u_char) 255))	/* LENGHT */
 	goto err;
-      stream_forward_endp (stream, 255);	/* VALUE */
+      stream_put (stream, NULL, 255);		/* zero padding */
     }
 
   left = STREAM_SIZE (stream) - stream_get_endp (stream);
@@ -1086,7 +1125,7 @@ tlv_add_padding (struct stream *stream)
 
   stream_putc (stream, PADDING);
   stream_putc (stream, left - 2);
-  stream_forward_endp (stream, left - 2);
+  stream_put (stream, NULL, left-2);
 
   return ISIS_OK;
 
