@@ -1,5 +1,5 @@
 /*
-   $Id: command.c,v 1.47 2005/04/25 16:26:42 paul Exp $
+   $Id: command.c,v 1.51 2005/10/26 05:49:54 paul Exp $
  
    Command interpreter routine for virtual terminal [aka TeletYpe]
    Copyright (C) 1997, 98, 99 Kunihiro Ishiguro
@@ -35,7 +35,7 @@ Boston, MA 02111-1307, USA.  */
 
 /* Command vector which includes some level of command lists. Normally
    each daemon maintains each own cmdvec. */
-vector cmdvec;
+vector cmdvec = NULL;
 
 /* Host information structure. */
 struct host host;
@@ -193,8 +193,8 @@ install_node (struct cmd_node *node,
 static int
 cmp_node (const void *p, const void *q)
 {
-  struct cmd_element *a = *(struct cmd_element **)p;
-  struct cmd_element *b = *(struct cmd_element **)q;
+  const struct cmd_element *a = *(struct cmd_element **)p;
+  const struct cmd_element *b = *(struct cmd_element **)q;
 
   return strcmp (a->string, b->string);
 }
@@ -202,8 +202,8 @@ cmp_node (const void *p, const void *q)
 static int
 cmp_desc (const void *p, const void *q)
 {
-  struct desc *a = *(struct desc **)p;
-  struct desc *b = *(struct desc **)q;
+  const struct desc *a = *(struct desc **)p;
+  const struct desc *b = *(struct desc **)q;
 
   return strcmp (a->cmd, b->cmd);
 }
@@ -475,7 +475,11 @@ void
 install_element (enum node_type ntype, struct cmd_element *cmd)
 {
   struct cmd_node *cnode;
-
+  
+  /* cmd_init hasn't been called */
+  if (!cmdvec)
+    return;
+  
   cnode = vector_slot (cmdvec, ntype);
 
   if (cnode == NULL) 
@@ -1886,7 +1890,7 @@ cmd_complete_command_real (vector vline, struct vty *vty, int *status)
 	    {
 	      char *lcdstr;
 
-	      lcdstr = XMALLOC (MTYPE_TMP, lcd + 1);
+	      lcdstr = XMALLOC (MTYPE_STRVEC, lcd + 1);
 	      memcpy (lcdstr, matchvec->index[0], lcd);
 	      lcdstr[lcd] = '\0';
 
@@ -1896,7 +1900,7 @@ cmd_complete_command_real (vector vline, struct vty *vty, int *status)
 	      for (i = 0; i < vector_active (matchvec); i++)
 		{
 		  if (vector_slot (matchvec, i))
-		    XFREE (MTYPE_TMP, vector_slot (matchvec, i));
+		    XFREE (MTYPE_STRVEC, vector_slot (matchvec, i));
 		}
 	      vector_free (matchvec);
 
@@ -1966,6 +1970,7 @@ node_parent ( enum node_type node )
     case BGP_IPV4_NODE:
     case BGP_IPV4M_NODE:
     case BGP_IPV6_NODE:
+    case BGP_IPV6M_NODE:
       ret = BGP_NODE;
       break;
     case KEYCHAIN_KEY_NODE:
@@ -2391,6 +2396,7 @@ DEFUN (config_exit,
     case BGP_IPV4_NODE:
     case BGP_IPV4M_NODE:
     case BGP_IPV6_NODE:
+    case BGP_IPV6M_NODE:
       vty->node = BGP_NODE;
       break;
     case KEYCHAIN_KEY_NODE:
@@ -2430,6 +2436,7 @@ DEFUN (config_end,
     case BGP_IPV4_NODE:
     case BGP_IPV4M_NODE:
     case BGP_IPV6_NODE:
+    case BGP_IPV6M_NODE:
     case RMAP_NODE:
     case OSPF_NODE:
     case OSPF6_NODE:
@@ -2517,6 +2524,7 @@ DEFUN (config_write_file,
   char *config_file;
   char *config_file_tmp = NULL;
   char *config_file_sav = NULL;
+  int ret = CMD_WARNING;
   struct vty *file_vty;
 
   /* Check and see if we are operating under vtysh configuration */
@@ -2530,12 +2538,13 @@ DEFUN (config_write_file,
   /* Get filename. */
   config_file = host.config;
   
-  config_file_sav = malloc (strlen (config_file) + strlen (CONF_BACKUP_EXT) + 1);
+  config_file_sav =
+    XMALLOC (MTYPE_TMP, strlen (config_file) + strlen (CONF_BACKUP_EXT) + 1);
   strcpy (config_file_sav, config_file);
   strcat (config_file_sav, CONF_BACKUP_EXT);
 
 
-  config_file_tmp = malloc (strlen (config_file) + 8);
+  config_file_tmp = XMALLOC (MTYPE_TMP, strlen (config_file) + 8);
   sprintf (config_file_tmp, "%s.XXXXXX", config_file);
   
   /* Open file to configuration write. */
@@ -2544,9 +2553,7 @@ DEFUN (config_write_file,
     {
       vty_out (vty, "Can't open configuration file %s.%s", config_file_tmp,
 	       VTY_NEWLINE);
-      free (config_file_tmp);
-      free (config_file_sav);
-      return CMD_WARNING;
+      goto finished;
     }
   
   /* Make vty for configuration file. */
@@ -2572,55 +2579,45 @@ DEFUN (config_write_file,
       {
 	vty_out (vty, "Can't unlink backup configuration file %s.%s", config_file_sav,
 		 VTY_NEWLINE);
-	free (config_file_sav);
-	free (config_file_tmp);
-	unlink (config_file_tmp);	
-	return CMD_WARNING;
+        goto finished;
       }
   if (link (config_file, config_file_sav) != 0)
     {
       vty_out (vty, "Can't backup old configuration file %s.%s", config_file_sav,
 	        VTY_NEWLINE);
-      free (config_file_sav);
-      free (config_file_tmp);
-      unlink (config_file_tmp);
-      return CMD_WARNING;
+      goto finished;
     }
   sync ();
   if (unlink (config_file) != 0)
     {
       vty_out (vty, "Can't unlink configuration file %s.%s", config_file,
 	        VTY_NEWLINE);
-      free (config_file_sav);
-      free (config_file_tmp);
-      unlink (config_file_tmp);
-      return CMD_WARNING;      
+      goto finished;
     }
   if (link (config_file_tmp, config_file) != 0)
     {
       vty_out (vty, "Can't save configuration file %s.%s", config_file,
 	       VTY_NEWLINE);
-      free (config_file_sav);
-      free (config_file_tmp);
-      unlink (config_file_tmp);
-      return CMD_WARNING;      
+      goto finished;
     }
-  unlink (config_file_tmp);
   sync ();
   
-  free (config_file_sav);
-  free (config_file_tmp);
-
   if (chmod (config_file, CONFIGFILE_MASK) != 0)
     {
       vty_out (vty, "Can't chmod configuration file %s: %s (%d).%s", 
 	config_file, safe_strerror(errno), errno, VTY_NEWLINE);
-      return CMD_WARNING;      
+      goto finished;
     }
 
   vty_out (vty, "Configuration saved to %s%s", config_file,
 	   VTY_NEWLINE);
-  return CMD_SUCCESS;
+  ret = CMD_SUCCESS;
+
+finished:
+  unlink (config_file_tmp);
+  XFREE (MTYPE_TMP, config_file_tmp);
+  XFREE (MTYPE_TMP, config_file_sav);
+  return ret;
 }
 
 ALIAS (config_write_file, 
@@ -2732,9 +2729,9 @@ DEFUN (config_hostname,
     }
 
   if (host.name)
-    XFREE (0, host.name);
+    XFREE (MTYPE_HOST, host.name);
     
-  host.name = strdup (argv[0]);
+  host.name = XSTRDUP (MTYPE_HOST, argv[0]);
   return CMD_SUCCESS;
 }
 
@@ -2746,7 +2743,7 @@ DEFUN (config_no_hostname,
        "Host name of this router\n")
 {
   if (host.name)
-    XFREE (0, host.name);
+    XFREE (MTYPE_HOST, host.name);
   host.name = NULL;
   return CMD_SUCCESS;
 }
@@ -2771,11 +2768,11 @@ DEFUN (config_password, password_cmd,
       if (*argv[0] == '8')
 	{
 	  if (host.password)
-	    XFREE (0, host.password);
+	    XFREE (MTYPE_HOST, host.password);
 	  host.password = NULL;
 	  if (host.password_encrypt)
-	    XFREE (0, host.password_encrypt);
-	  host.password_encrypt = XSTRDUP (0, strdup (argv[1]));
+	    XFREE (MTYPE_HOST, host.password_encrypt);
+	  host.password_encrypt = XSTRDUP (MTYPE_HOST, argv[1]);
 	  return CMD_SUCCESS;
 	}
       else
@@ -2793,17 +2790,17 @@ DEFUN (config_password, password_cmd,
     }
 
   if (host.password)
-    XFREE (0, host.password);
+    XFREE (MTYPE_HOST, host.password);
   host.password = NULL;
 
   if (host.encrypt)
     {
       if (host.password_encrypt)
-	XFREE (0, host.password_encrypt);
-      host.password_encrypt = XSTRDUP (0, zencrypt (argv[0]));
+	XFREE (MTYPE_HOST, host.password_encrypt);
+      host.password_encrypt = XSTRDUP (MTYPE_HOST, zencrypt (argv[0]));
     }
   else
-    host.password = XSTRDUP (0, argv[0]);
+    host.password = XSTRDUP (MTYPE_HOST, argv[0]);
 
   return CMD_SUCCESS;
 }
@@ -2835,12 +2832,12 @@ DEFUN (config_enable_password, enable_password_cmd,
       if (*argv[0] == '8')
 	{
 	  if (host.enable)
-	    XFREE (0, host.enable);
+	    XFREE (MTYPE_HOST, host.enable);
 	  host.enable = NULL;
 
 	  if (host.enable_encrypt)
-	    XFREE (0, host.enable_encrypt);
-	  host.enable_encrypt = XSTRDUP (0, argv[1]);
+	    XFREE (MTYPE_HOST, host.enable_encrypt);
+	  host.enable_encrypt = XSTRDUP (MTYPE_HOST, argv[1]);
 
 	  return CMD_SUCCESS;
 	}
@@ -2859,18 +2856,18 @@ DEFUN (config_enable_password, enable_password_cmd,
     }
 
   if (host.enable)
-    XFREE (0, host.enable);
+    XFREE (MTYPE_HOST, host.enable);
   host.enable = NULL;
 
   /* Plain password input. */
   if (host.encrypt)
     {
       if (host.enable_encrypt)
-	XFREE (0, host.enable_encrypt);
-      host.enable_encrypt = XSTRDUP (0, zencrypt (argv[0]));
+	XFREE (MTYPE_HOST, host.enable_encrypt);
+      host.enable_encrypt = XSTRDUP (MTYPE_HOST, zencrypt (argv[0]));
     }
   else
-    host.enable = XSTRDUP (0, argv[0]);
+    host.enable = XSTRDUP (MTYPE_HOST, argv[0]);
 
   return CMD_SUCCESS;
 }
@@ -2890,11 +2887,11 @@ DEFUN (no_config_enable_password, no_enable_password_cmd,
        "Assign the privileged level password\n")
 {
   if (host.enable)
-    XFREE (0, host.enable);
+    XFREE (MTYPE_HOST, host.enable);
   host.enable = NULL;
 
   if (host.enable_encrypt)
-    XFREE (0, host.enable_encrypt);
+    XFREE (MTYPE_HOST, host.enable_encrypt);
   host.enable_encrypt = NULL;
 
   return CMD_SUCCESS;
@@ -2914,14 +2911,14 @@ DEFUN (service_password_encrypt,
   if (host.password)
     {
       if (host.password_encrypt)
-	XFREE (0, host.password_encrypt);
-      host.password_encrypt = XSTRDUP (0, zencrypt (host.password));
+	XFREE (MTYPE_HOST, host.password_encrypt);
+      host.password_encrypt = XSTRDUP (MTYPE_HOST, zencrypt (host.password));
     }
   if (host.enable)
     {
       if (host.enable_encrypt)
-	XFREE (0, host.enable_encrypt);
-      host.enable_encrypt = XSTRDUP (0, zencrypt (host.enable));
+	XFREE (MTYPE_HOST, host.enable_encrypt);
+      host.enable_encrypt = XSTRDUP (MTYPE_HOST, zencrypt (host.enable));
     }
 
   return CMD_SUCCESS;
@@ -2940,11 +2937,11 @@ DEFUN (no_service_password_encrypt,
   host.encrypt = 0;
 
   if (host.password_encrypt)
-    XFREE (0, host.password_encrypt);
+    XFREE (MTYPE_HOST, host.password_encrypt);
   host.password_encrypt = NULL;
 
   if (host.enable_encrypt)
-    XFREE (0, host.enable_encrypt);
+    XFREE (MTYPE_HOST, host.enable_encrypt);
   host.enable_encrypt = NULL;
 
   return CMD_SUCCESS;
@@ -3213,9 +3210,9 @@ set_log_file(struct vty *vty, const char *fname, int loglevel)
     }
 
   if (host.logfile)
-    XFREE (MTYPE_TMP, host.logfile);
+    XFREE (MTYPE_HOST, host.logfile);
 
-  host.logfile = XSTRDUP (MTYPE_TMP, fname);
+  host.logfile = XSTRDUP (MTYPE_HOST, fname);
 
   return CMD_SUCCESS;
 }
@@ -3256,7 +3253,7 @@ DEFUN (no_config_log_file,
   zlog_reset_file (NULL);
 
   if (host.logfile)
-    XFREE (MTYPE_TMP, host.logfile);
+    XFREE (MTYPE_HOST, host.logfile);
 
   host.logfile = NULL;
 
@@ -3425,8 +3422,8 @@ DEFUN (banner_motd_file,
        "Filename\n")
 {
   if (host.motdfile)
-    XFREE (MTYPE_TMP, host.motdfile);
-  host.motdfile = XSTRDUP (MTYPE_TMP, argv[0]);
+    XFREE (MTYPE_HOST, host.motdfile);
+  host.motdfile = XSTRDUP (MTYPE_HOST, argv[0]);
 
   return CMD_SUCCESS;
 }
@@ -3451,7 +3448,7 @@ DEFUN (no_banner_motd,
 {
   host.motd = NULL;
   if (host.motdfile) 
-    XFREE (MTYPE_TMP, host.motdfile);
+    XFREE (MTYPE_HOST, host.motdfile);
   host.motdfile = NULL;
   return CMD_SUCCESS;
 }
@@ -3460,7 +3457,7 @@ DEFUN (no_banner_motd,
 void
 host_config_set (char *filename)
 {
-  host.config = strdup (filename);
+  host.config = XSTRDUP (MTYPE_HOST, filename);
 }
 
 void
