@@ -68,12 +68,10 @@ ospf_router_id_update_zebra (int command, struct zclient *zclient,
   router_id_zebra = router_id.u.prefix4;
 
   ospf = ospf_lookup ();
+  
   if (ospf != NULL)
-    {
-      if (ospf->t_router_id_update == NULL)
-	OSPF_TIMER_ON (ospf->t_router_id_update, ospf_router_id_update_timer,
-                       OSPF_ROUTER_ID_UPDATE_DELAY);
-    }
+    ospf_router_id_update (ospf);
+  
   return 0;
 }
 
@@ -291,7 +289,10 @@ ospf_interface_address_delete (int command, struct zclient *zclient,
 
   rn = route_node_lookup (IF_OIFS (ifp), &p);
   if (!rn)
-    return 0;
+    {
+      connected_free (c);
+      return 0;
+    }
 
   assert (rn->info);
   oi = rn->info;
@@ -341,11 +342,8 @@ ospf_zebra_add (struct prefix_ipv4 *p, struct ospf_route *or)
       s = zclient->obuf;
       stream_reset (s);
 
-      /* Length place holder. */
-      stream_putw (s, 0);
-
       /* Put command, type, flags, message. */
-      stream_putc (s, ZEBRA_IPV4_ROUTE_ADD);
+      zclient_create_header (s, ZEBRA_IPV4_ROUTE_ADD);
       stream_putc (s, ZEBRA_ROUTE_OSPF);
       stream_putc (s, flags);
       stream_putc (s, message);
