@@ -286,6 +286,36 @@ static struct test_segment {
       "8466 3 52737 4096 34285 8466 3 52737 4096 34285",
       250, 0, NOT_ALL_PRIVATE, 4096, 4, 8466 },
   },
+  { /* 15 */ 
+    "seq1extra",
+    "seq(8466,3,52737,4096,3456)",
+    { 0x2,0x5, 0x21,0x12, 0x00,0x03, 0xce,0x01, 0x10,0x00, 0x0d,0x80 },
+    12,
+    { "8466 3 52737 4096 3456",
+      "8466 3 52737 4096 3456",
+      5, 0, NOT_ALL_PRIVATE, 4096, 4, 8466 },
+  },
+  { /* 16 */
+    "empty",
+    "<empty>",
+    {},
+    0,
+    { "", "", 0, 0, 0, 0, 0, 0 },
+  },
+  { /* 17 */ 
+    "redundantset",
+    "seq(8466,3,52737,4096,3456) set(7099,8153,8153,8153)",
+    { 0x2,0x5, 0x21,0x12, 0x00,0x03, 0xce,0x01, 0x10,0x00, 0x0d,0x80,
+      0x1,0x4, 0x1b,0xbb, 0x1f,0xd9, 0x1f,0xd9, 0x1f,0xd9 },
+    22,
+    {
+     /* We shouldn't ever /generate/ such paths. However, we should
+      * cope with them fine.
+      */
+     "8466 3 52737 4096 3456 {7099,8153,8153,8153}",
+      "8466 3 52737 4096 3456 {7099,8153,8153,8153}",
+      6, 0, NOT_ALL_PRIVATE, 4096, 4, 8466 },
+  },
   { NULL, NULL, {0}, 0, { NULL, 0, 0 } }
 };
 
@@ -445,6 +475,8 @@ struct compare_tests
   { 0, 1, CMP_RES_NO, CMP_RES_NO },
   { 0, 2, CMP_RES_YES, CMP_RES_NO },
   { 0, 11, CMP_RES_YES, CMP_RES_NO },
+  { 0, 15, CMP_RES_YES, CMP_RES_NO },
+  { 0, 16, CMP_RES_NO, CMP_RES_NO },
   { 1, 11, CMP_RES_NO, CMP_RES_NO },
   { 6, 7, CMP_RES_NO, CMP_RES_YES },
   { 6, 8, CMP_RES_NO, CMP_RES_NO },
@@ -477,7 +509,9 @@ make_aspath (const u_char *data, size_t len)
       stream_put (s, data, len);
     }
   as = aspath_parse (s, len);
-  stream_free (s);
+  
+  if (s)
+    stream_free (s);
   
   return as;
 }
@@ -601,7 +635,9 @@ validate (struct aspath *as, const struct test_spec *sp)
               aspath_private_as_check (as));
     }
   aspath_unintern (asinout);
- /* aspath_unintern (asconfeddel);*/
+  aspath_free (asconfeddel);
+  aspath_free (asstr);
+  
   return fails;
   
 }
@@ -619,6 +655,8 @@ empty_get_test ()
     printf ("failed!\n");
   
   printf ("\n");
+  
+  aspath_free (as);
 }
 
 /* basic parsing test */
@@ -643,7 +681,7 @@ parse_test (struct test_segment *t)
 }
 
 /* prepend testing */
-void
+static void
 prepend_test (struct tests *t)
 {
   struct aspath *asp1, *asp2, *ascratch;
@@ -672,7 +710,7 @@ prepend_test (struct tests *t)
 }
 
 /* empty-prepend testing */
-void
+static void
 empty_prepend_test (struct test_segment *t)
 {
   struct aspath *asp1, *asp2, *ascratch;
@@ -700,7 +738,7 @@ empty_prepend_test (struct test_segment *t)
 }
 
 /* aggregation testing */
-void
+static void
 aggregate_test (struct tests *t)
 {
   struct aspath *asp1, *asp2, *ascratch;
